"""Supported Versions class."""


class SupportedVersions:
    """Container for all supported versions by the ONYX.CENTER."""

    def __init__(self, versions: list):
        """Initialize the versions.

        versions: a list of supported versions"""
        self.versions = versions

    def supports(self, version: str) -> bool:
        """Check if the provided version is supported.

        version: the version to check for"""
        return version in self.versions



"""Onyx Client authorizer."""

import logging
from typing import Optional

import aiohttp

from onyx_client.client import OnyxClient
from onyx_client.configuration.configuration import Configuration
from onyx_client.utils.const import API_URL, API_HEADERS
from onyx_client.utils.response import check

_LOGGER = logging.getLogger(__name__)


async def exchange_code(
    code: str, client_session: aiohttp.ClientSession = None
) -> Optional[Configuration]:
    """Exchange an API code for an access token and fingerprint.

    code: the access code
    client_session: the aiohttp client session to use"""
    session = client_session if client_session is not None else aiohttp.ClientSession()
    config = await authorize(code, session)
    if await OnyxClient(config, session).verify():
        return config
    return None


async def authorize(
    code: str, client_session: aiohttp.ClientSession
) -> Optional[Configuration]:
    """Authorize the client using an API code.

    code: the access code
    client_session: the aiohttp client session to use"""
    async with client_session.post(
        f"{API_URL}/authorize",
        data={"code": code},
        headers=API_HEADERS,
    ) as response:
        if not check(response):
            _LOGGER.error("Could not authorize client for ONYX API.")
            return None
        data = await response.json()
        return Configuration(data.get("fingerprint", None), data.get("token"))



"""Numeric Values of Onyx devices."""
from typing import Optional

from onyx_client.data.animation_value import AnimationValue


class NumericValue:
    """The representation of a numeric value."""

    def __init__(
        self,
        value: int,
        minimum: int,
        maximum: int,
        read_only: bool,
        animation: Optional[AnimationValue] = None,
    ):
        """Initialize the numeric value.

        value: the value
        minimum: the minimum supported value
        maximum: the maximum supported value
        read_only: set of the value is read only
        animation: an optional ongoing animation"""
        self.value = value
        self.minimum = minimum
        self.maximum = maximum
        self.read_only = read_only
        self.animation = animation

    def __str__(self) -> str:
        return f"NumericValue(value={self.value}, minimum={self.minimum}, maximum={self.maximum}, animation={self.animation})"

    @staticmethod
    def create(properties: dict):
        """Create a numeric value from properties.

        properties: the properties of the device"""
        if properties is None:
            return None

        return NumericValue(
            properties.get("value", None),
            properties.get("minimum", None),
            properties.get("maximum", None),
            properties.get("read_only", False),
            AnimationValue.create(properties.get("animation", None)),
        )

    def update_with(self, other: Optional["NumericValue"]):
        """Updates this value with the target.

        other: the other value"""
        if other is not None:
            self.value = self.value if other.value is None else other.value
            self.minimum = self.minimum if other.minimum is None else other.minimum
            self.maximum = self.maximum if other.maximum is None else other.maximum
            self.read_only = (
                self.read_only if other.read_only is None else other.read_only
            )
            # if other.animation is None it would mean that the animation is cancelled by Onyx.
            # in fact, we can just overwrite it no matter the value.
            self.animation = other.animation

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return (
                self.value == other.value
                and self.minimum == other.minimum
                and self.maximum == other.maximum
                and self.read_only == other.read_only
                and self.animation == other.animation
            )
        return False



"""Onyx Client device type utils."""
from typing import Optional

from onyx_client.device.click import Click
from onyx_client.device.light import Light
from onyx_client.device.shutter import Shutter
from onyx_client.device.switch import Switch
from onyx_client.device.weather import Weather
from onyx_client.enum.device_type import DeviceType


def is_shutter(device_type: DeviceType, properties: dict) -> bool:
    """Check if the provided device type is a shutter.

    device_type: the device type to check
    properties: the property map to check if no explicit type is specified"""
    if device_type is not None:
        return device_type.is_shutter()
    return _in_keys(properties, Shutter.keys())


def is_light(device_type: DeviceType, properties: dict) -> bool:
    """Check if the provided device type is a light.

    device_type: the device type to check
    properties: the property map to check if no explicit type is specified"""
    if device_type is not None:
        return device_type.is_light()
    return _in_keys(properties, Light.keys())


def is_weather(device_type: DeviceType, properties: dict) -> bool:
    """Check if the provided device type is a weather station.

    device_type: the device type to check
    properties: the property map to check if no explicit type is specified"""
    if device_type is not None:
        return device_type == DeviceType.WEATHER
    return _in_keys(properties, Weather.keys())


def is_click(device_type: DeviceType, data: dict) -> bool:
    """Check if the provided device type is a click device.

    device_type: the device type to check
    properties: the property map to check if no explicit type is specified"""
    if device_type is not None:
        return device_type == DeviceType.CLICK
    return _in_keys(data, Click.keys())


def is_switch(device_type: DeviceType, data: dict) -> bool:
    """Check if the provided device type is a switch.

    device_type: the device type to check
    properties: the property map to check if no explicit type is specified"""
    if device_type is not None:
        return device_type == DeviceType.SWITCH
    return _in_keys(data, Switch.keys())


def _in_keys(data: Optional[dict], keys: list) -> bool:
    """Check if any key in data is listed in the keys list.

    data: the data map to check
    keys: the keys to check for"""
    if data is not None:
        for key in data.keys():
            if key in keys:
                return True
    return False



"""Onyx Client response utils."""
import logging

import aiohttp

_LOGGER = logging.getLogger(__name__)


def check(response: aiohttp.ClientResponse) -> bool:
    """Check the response for a success HTTP status code.

    Success codes are:
      - 200

    response: the response to check"""
    if response.status == 200:
        _LOGGER.debug("Received HTTP response from ONYX API: %s", response.status)
        return True
    else:
        _LOGGER.error(
            "Received erroneous HTTP response from ONYX API: %s", response.status
        )
        return False



"""Click class."""
from onyx_client.data.device_mode import DeviceMode
from onyx_client.device.device import Device
from onyx_client.enum.device_type import DeviceType


class Click(Device):
    """A ONYX controlled click device."""

    def __init__(
        self, identifier: str, name: str, device_type: DeviceType, offline: bool
    ):
        """Initialize the click device.

        identifier: the device identifier
        name: the device name
        device_type: the device type
        offline: set if the device is offline"""
        super(Click, self).__init__(
            identifier,
            name,
            device_type,
            DeviceMode(device_type),
            list(),
        )
        self.offline = offline

    def __str__(self):
        return f"Click({super().__str__()}, offline={self.offline})"

    def update_with(self, update):
        """Update the device with an update patch.

        update: the update patch"""
        super().update_with(update)

        self.offline = self.offline if update.offline is None else update.offline

    @staticmethod
    def keys() -> list:
        """Get the list of keys specific to the device type."""
        return [
            "offline",
        ]



"""Onyx Client API class."""
import json
import logging
from typing import Optional
from random import uniform

import aiohttp
import asyncio

from onyx_client.configuration.configuration import Configuration
from onyx_client.data.date_information import DateInformation
from onyx_client.data.device_command import DeviceCommand
from onyx_client.data.supported_versions import SupportedVersions
from onyx_client.device.device import Device
from onyx_client.enum.action import Action
from onyx_client.enum.device_type import DeviceType
from onyx_client.group.group import Group
from onyx_client.helpers.url import UrlHelper
from onyx_client.utils.const import API_VERSION
from onyx_client.utils.filter import present
from onyx_client.utils.mapper import init_device

_LOGGER = logging.getLogger(__name__)


class OnyxClient:
    """The ONYX.CENTER API Client.

    After initializing, call ::verify to check if:
      - the provided connection parameters are correct
      - the ONYX.CENTER supports the client's API version"""

    def __init__(
        self,
        config: Configuration,
        client_session: aiohttp.ClientSession,
        event_loop=asyncio.get_event_loop(),
    ):
        """Initialize the API client.

        config: the access configuration of the client
        client_session: the aiohttp session to use
        event_loop: the event loop to use for background events"""
        self.config = config
        self.url_helper = UrlHelper(config, client_session)
        self._shutdown = True
        self._read_loop_task = None
        self._event_loop = event_loop
        self._active_tasks = set()
        self._event_callback = None

    async def supported_versions(self) -> Optional[SupportedVersions]:
        """Get all supported versions by the ONYX.CENTER."""
        data = await self.url_helper.perform_get_request("/versions", with_api=False)
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /versions.",
                self.config.fingerprint,
            )
            return None

        return SupportedVersions(data.get("versions", list()))

    async def verify(self) -> bool:
        """Check if the ONYX.CENTER supports the version
        and the connection parameters are working."""
        versions = await self.supported_versions()
        return versions.supports(API_VERSION) if versions is not None else False

    async def date_information(self) -> Optional[DateInformation]:
        """Get all date related information of the ONYX.CENTER."""
        data = await self.url_helper.perform_get_request("/clock")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /clock.",
                self.config.fingerprint,
            )
            return None

        return DateInformation(
            float(data.get("time", "0")),
            data.get("zone", None),
            int(data.get("zone_offset", "0")),
        )

    async def devices(self, include_details: bool = False) -> Optional[list]:
        """Get all devices controlled by the ONYX.CENTER.

        include_details: ensures all device details are queried
                         before returning the device"""
        data = await self.url_helper.perform_get_request("/devices")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices.",
                self.config.fingerprint,
            )
            return None

        if include_details:
            return [
                device
                for device in [await self.device(key) for key, _ in data.items()]
                if present(device)
            ]
        else:
            return [
                init_device(
                    key,
                    value.get("name", None),
                    DeviceType.convert(value.get("type", None)),
                )
                for key, value in data.items()
            ]

    async def device(self, identifier: str) -> Optional[Device]:
        """Get the device properties for a provided ID.

        identifier: the identifier of the device to query"""
        data = await self.url_helper.perform_get_request(f"/devices/{identifier}")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices/%s.",
                self.config.fingerprint,
                identifier,
            )
            return None

        actions = [Action.convert(action) for action in data.get("actions", list())]
        return init_device(
            identifier,
            data.get("name", None),
            DeviceType.convert(data.get("type", None)),
            data.get("properties", None),
            actions,
            data,
        )

    async def send_command(self, identifier: str, command: DeviceCommand) -> bool:
        """Send a command to the device with the provided ID.

        identifier: the device identifier
        command: the command object to send to the device"""
        data = await self.url_helper.perform_post_request(
            f"/devices/{identifier}/command", command.data()
        )
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices/%s/command.",
                self.config.fingerprint,
                identifier,
            )
        return data is not None

    async def cancel_command(self, identifier: str) -> bool:
        """Cancel a command to the device with the provided ID.

        identifier: the device identifier to cancel the command for"""
        data = await self.url_helper.perform_delete_request(
            f"/devices/{identifier}/command"
        )
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /devices/%s/command.",
                self.config.fingerprint,
                identifier,
            )
        return data is not None

    async def groups(self) -> Optional[list]:
        """Get all groups controlled by the ONYX.CENTER."""
        data = await self.url_helper.perform_get_request("/groups")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups.",
                self.config.fingerprint,
            )
            return None

        return [
            Group(key, value.get("name", None), value.get("devices", list()))
            for key, value in data.items()
        ]

    async def group(self, identifier: str) -> Optional[Group]:
        """Get the group properties for a provided ID.

        identifier: the group identifier to query"""
        data = await self.url_helper.perform_get_request(f"/groups/{identifier}")
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups/%s.",
                self.config.fingerprint,
                identifier,
            )
            return None

        return Group(identifier, data.get("name", None), data.get("devices", list()))

    async def send_group_command(self, identifier: str, command: DeviceCommand) -> bool:
        """Send a command to the group with the provided ID.

        identifier: the group identifier
        command: the command object to send to the group"""
        data = await self.url_helper.perform_post_request(
            f"/groups/{identifier}/command", command.data()
        )
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups/%s/command.",
                self.config.fingerprint,
                identifier,
            )
            return False

        unsuccessful = [
            key
            for (key, value) in data.get("results", dict()).items()
            if value.get("status_code", 501) != 200
        ]
        if len(unsuccessful) > 0:
            _LOGGER.error(
                "Could not execute command for all devices in group %s: %s",
                identifier,
                unsuccessful,
            )
        return len(unsuccessful) == 0

    async def cancel_group_command(self, identifier: str) -> bool:
        """Cancel a command to the group with the provided ID.

        identifier: the group identifier to cancel the command for"""
        data = await self.url_helper.perform_delete_request(
            f"/groups/{identifier}/command"
        )
        if data is None:
            _LOGGER.error(
                "Could not call ONYX API for device %s: /groups/%s/command.",
                self.config.fingerprint,
                identifier,
            )
        return data is not None

    async def events(self, include_details: bool = False) -> Device:
        """Stream events continuously.

        include_details: ensures all device details are queried
                         before emiting the device"""
        async for message in self.url_helper.start_stream("/events"):
            if message is not None and len(message) > 0 and message.startswith("data:"):
                message = message[len("data:") :].strip()
                events = json.loads(message)
                for key, value in events.get("devices", dict()).items():
                    try:
                        if value is not None:
                            device = (
                                await self.device(key)
                                if include_details
                                else init_device(
                                    key,
                                    value.get("name", None),
                                    DeviceType.convert(value.get("type", None)),
                                    value.get("properties", None),
                                    value,
                                )
                            )
                            yield device
                    except AttributeError:
                        _LOGGER.error(
                            "Received unknown device data. Dropping device %s", key
                        )

    def start(self, include_details: bool = False, backoff_time: int = 1):
        """Start the event stream via callback.

        include_details: ensures all device details are queried
                         before emiting the device
        backoff_time: the maximum time in minutes for a connection retry"""
        self._shutdown = False
        self._read_loop_task = self._create_internal_task(
            self._read_handler(include_details, backoff_time), name="read_loop"
        )

    def stop(self):
        """Stop the event stream via callback."""
        self._shutdown = True

    def set_event_callback(self, callback):
        """Set the event stream callback.

        callback: the callback function taking the device as the only parameter"""
        self._event_callback = callback

    def _create_internal_task(self, coro, name=None):
        """Create an internal task running in the background.

        coro: the coroutine to run
        name: the event loop name"""
        task = self._event_loop.create_task(coro, name=name)
        task.add_done_callback(self._complete_internal_task)
        self._active_tasks.add(task)

    async def _read_handler(self, include_details: bool = False, backoff_time: int = 1):
        """Handle rerunning the task in the background.

        include_details: ensures all device details are queried
                         before emiting the device
        backoff_time: the maximum time in minutes for a connection retry"""
        while not self._shutdown:
            try:
                await self._read_loop(include_details)
            except asyncio.CancelledError:
                raise
            except Exception as ex:
                backoff = int(uniform(0, backoff_time) * 60)
                _LOGGER.error(
                    "Unexpected exception: %r. Retrying with backoff %ds.", ex, backoff
                )
                await asyncio.sleep(backoff)

    def _complete_internal_task(self, task):
        """Remove an internal task that was running in the background.

        task: the task to remove"""
        self._active_tasks.remove(task)
        if not task.cancelled():
            ex = task.exception()
            _LOGGER.error("Unexpected exception: %r. Completing task.", ex)
            raise ex

    async def _read_loop(self, include_details: bool = False):
        """Streams data from the ONYX API endpoint and emits device updates.
        Updates are emitted as events through the event_callback.

        include_details: ensures all device details are queried
                         before emiting the device"""
        while not self._shutdown:
            async for device in self.events(include_details):
                if self._shutdown:
                    break
                if self._event_callback is not None:
                    _LOGGER.debug("Received device: %s", device)
                    self._event_callback(device)
                else:
                    _LOGGER.warning("Received data but no callback is defined.")


def create(
    config: Configuration = None,
    fingerprint: str = None,
    access_token: str = None,
    client_session: aiohttp.ClientSession = None,
    event_loop=None,
) -> OnyxClient:
    """Create the client.

    Either config or fingerprint and access_token must be provided.

    config: the access configuration of the client (optional)
    fingerprint: the ONYX.CENTER fingerprint (optional)
    access_token: the access token to use (optional)
    client_session: the aiohttp session to use
    event_loop: the event loop to use for background events"""
    if config is None:
        config = Configuration(fingerprint, access_token)
    session = client_session if client_session is not None else aiohttp.ClientSession()
    event_loop = event_loop if event_loop is not None else asyncio.get_event_loop()
    return OnyxClient(config, session, event_loop)



"""Device Types of Onyx devices."""
from enum import Enum, auto


class DeviceType(Enum):
    """The device types supported by Onyx."""

    ROLLERSHUTTER = auto()
    AWNING = auto()
    RAFFSTORE_90 = auto()
    RAFFSTORE_180 = auto()
    WEATHER = auto()
    VENEER = auto()
    BASIC_LIGHT = auto()
    CLICK = auto()
    SWITCH = auto()
    PERGOLA_AWNING_ROOF = auto()
    PERGOLA_SIDE = auto()
    PERGOLA_SLAT_ROOF = auto()
    DIMMABLE_LIGHT = auto()
    UNKNOWN = 9999

    def string(self) -> str:
        """Get the string representation for the Onyx API."""
        return self.name.lower()

    def is_shutter(self) -> bool:
        """Check if the type corresponds to any kind of shutter."""
        return self in [
            self.ROLLERSHUTTER,
            self.AWNING,
            self.RAFFSTORE_90,
            self.RAFFSTORE_180,
            self.VENEER,
            self.PERGOLA_AWNING_ROOF,
            self.PERGOLA_SIDE,
            self.PERGOLA_SLAT_ROOF,
        ]

    def is_light(self) -> bool:
        """Check if the type corresponds to any kind of light."""
        return self in [
            self.BASIC_LIGHT,
            self.DIMMABLE_LIGHT,
        ]

    @staticmethod
    def convert(lower: str):
        """Get the device type from the Onyx API's type information.

        lower: the lower case device type identifier"""
        try:
            return DeviceType[lower.upper()]
        except (KeyError, AttributeError):
            return None



"""Onyx Client mapper utils."""
from onyx_client.data.boolean_value import BooleanValue
from onyx_client.data.device_mode import DeviceMode
from onyx_client.data.numeric_value import NumericValue
from onyx_client.device.click import Click
from onyx_client.device.device import Device
from onyx_client.device.light import Light
from onyx_client.device.shutter import Shutter
from onyx_client.device.switch import Switch
from onyx_client.device.weather import Weather
from onyx_client.enum.device_type import DeviceType
from onyx_client.utils.device_type import (
    is_click,
    is_light,
    is_weather,
    is_shutter,
    is_switch,
)


def numeric_value(key: str, properties: dict = None):
    """Create a new numeric value.

    key: the key value
    properties: the device properties"""
    return (
        NumericValue.create(properties.get(key, None))
        if properties is not None
        else None
    )


def boolean_value(key: str, properties: dict = None):
    """Create a new boolean value.

    key: the key value
    properties: the device properties"""
    return (
        BooleanValue.create(properties.get(key, None))
        if properties is not None
        else None
    )


def init_device(
    identifier: str,
    name: str = None,
    device_type: DeviceType = None,
    properties: dict = None,
    actions: list = None,
    data: dict = None,
) -> Device:
    """Initialize the device correctly.

    identifier: the device identifier
    name: the device name
    device_type: the device type
    properties: the properties of the device to set
    actions: a list of actions the device supports
    data: the data map of the device"""
    device_mode_value = (
        DeviceType.convert(properties.get("device_type", dict()).get("type", None))
        if properties is not None
        else device_type
    )
    device_mode_values = (
        [
            DeviceType.convert(value)
            for value in properties.get("device_type", dict()).get("values", list())
        ]
        if properties is not None
        else None
    )
    device_mode = DeviceMode(device_mode_value, device_mode_values)
    if is_shutter(device_type, properties):
        return Shutter(
            identifier,
            name,
            device_type,
            device_mode,
            actions,
            numeric_value("target_position", properties),
            numeric_value("target_angle", properties),
            numeric_value("actual_angle", properties),
            numeric_value("actual_position", properties),
        )
    elif is_weather(device_type, properties):
        return Weather(
            identifier,
            name,
            device_type,
            device_mode,
            actions,
            numeric_value("wind_peak", properties),
            numeric_value("sun_brightness_peak", properties),
            numeric_value("sun_brightness_sink", properties),
            numeric_value("air_pressure", properties),
            numeric_value("humidity", properties),
            numeric_value("temperature", properties),
        )
    elif is_light(device_type, properties):
        return Light(
            identifier,
            name,
            device_type,
            device_mode,
            actions,
            numeric_value("target_brightness", properties),
            numeric_value("actual_brightness", properties),
            numeric_value("dim_duration", properties),
        )
    elif is_click(device_type, data):
        offline = data.get("offline", True) if data is not None else True
        return Click(identifier, name, device_type, offline)
    elif is_switch(device_type, data):
        return Switch(identifier, name, device_type)
    else:
        return Device(identifier, name, DeviceType.UNKNOWN, device_mode, actions)



"""Animation Values of Onyx devices."""
from typing import Optional

from onyx_client.data.animation_keyframe import AnimationKeyframe


class AnimationValue:
    """The representation of an animation value."""

    def __init__(self, start: float, current_value: int, keyframes: list):
        """Initialize the animation value.

        start: the start value
        current_value: the current value in the animation
        keyframes: the list of keyframes until the end of the animation"""
        self.start = start
        self.current_value = current_value
        self.keyframes = keyframes

    def __str__(self) -> str:
        return f"AnimationValue(start={self.start}, current_value={self.current_value}, #keyframes={len(self.keyframes)})"

    @staticmethod
    def create(properties: dict):
        """Create an animation value from properties.

        properties: the properties of the device"""
        if properties is None:
            return None

        return AnimationValue(
            properties.get("start", None),
            properties.get("current_value"),
            [
                AnimationKeyframe.create(keyframe)
                for keyframe in properties.get("keyframes", list())
            ],
        )

    def update_with(self, other: Optional["AnimationValue"]):
        """Updates this value with the target.

        other: the other value"""
        if other is not None:
            self.start = self.start if other.start is None else other.start
            self.current_value = (
                self.current_value
                if other.current_value is None
                else other.current_value
            )
            self.keyframes = (
                self.keyframes if other.keyframes is None else other.keyframes
            )

    def __eq__(self, other):
        if isinstance(self, other.__class__):
            return (
                self.start == other.start and self.current_value == other.current_value
            )
        return False



"""Light class."""
from typing import Optional

from onyx_client.data.device_mode import DeviceMode
from onyx_client.data.numeric_value import NumericValue
from onyx_client.device.device import Device
from onyx_client.enum.device_type import DeviceType


class Light(Device):
    """A ONYX controlled light."""

    def __init__(
        self,
        identifier: str,
        name: str,
        device_type: DeviceType,
        device_mode: DeviceMode,
        actions: list,
        target_brightness: NumericValue = None,
        actual_brightness: NumericValue = None,
        dim_duration: NumericValue = None,
    ):
        """Initialize the light device.

        identifier: the device identifier
        name: the device name
        device_type: the device type
        device_mode: the mode the device can operate as
        actions: a list of actions the device supports
        target_brightness: the target brightness of the light
        actual_brightness: the actual brightness of the light
        dim_duration: the duration it takes to dim"""
        super(Light, self).__init__(identifier, name, device_type, device_mode, actions)
        self.target_brightness = target_brightness
        self.actual_brightness = actual_brightness
        self.dim_duration = dim_duration

    def __str__(self):
        return f"Light({super().__str__()}, actual_brightness={self.actual_brightness}, target_brightness={self.target_brightness}, dim_duration={self.dim_duration})"

    def update_with(self, update: Optional["Light"]):
        """Update the device with a patch.

        update: the update patch"""
        super().update_with(update)

        if self.target_brightness is not None:
            self.target_brightness.update_with(update.target_brightness)
        else:
            self.target_brightness = update.target_brightness
        self.actual_brightness.update_with(update.actual_brightness)
        self.dim_duration.update_with(update.dim_duration)

    @staticmethod
    def keys() -> list:
        """Get the list of keys specific to the device type."""
        return [
            "target_brightness",
            "actual_brightness",
            "dim_duration",
        ]
